@@ -1,7 +1,7 @@
 /*
  * Copyright © 2001-2011 Stéphane Raimbault <stephane.raimbault@gmail.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifndef MODBUS_TCP_PRIVATE_H
@@ -14,7 +14,7 @@
 #define _MODBUS_TCP_CHECKSUM_LENGTH    0
 
 /* In both structures, the transaction ID must be placed on first position
-   to have a quick access not dependant of the TCP backend */
+   to have a quick access not dependent of the TCP backend */
 typedef struct _modbus_tcp {
     /* Extract from MODBUS Messaging on TCP/IP Implementation Guide V1.0b
        (page 23/46):
@@ -27,18 +27,15 @@ typedef struct _modbus_tcp {
     char ip[16];
 } modbus_tcp_t;
 
-#define _MODBUS_TCP_PI_NODE_LENGTH    1025
-#define _MODBUS_TCP_PI_SERVICE_LENGTH   32
-
 typedef struct _modbus_tcp_pi {
     /* Transaction ID */
     uint16_t t_id;
     /* TCP port */
     int port;
     /* Node */
-    char node[_MODBUS_TCP_PI_NODE_LENGTH];
+    char *node;
     /* Service */
-    char service[_MODBUS_TCP_PI_SERVICE_LENGTH];
+    char *service;
 } modbus_tcp_pi_t;
 
 #endif /* MODBUS_TCP_PRIVATE_H */
